unit md5;

INTERFACE

uses Windows;

type
        MD5Count   = array[0..1] of DWORD;
        MD5State   = array[0..3] of DWORD;
        MD5Block   = array[0..15] of DWORD;
        MD5CBits   = array[0..7] of byte;
        MD5Digest  = array[0..15] of byte;
        MD5Buffer  = array[0..63] of byte;
        MD5Context = record
                      State: MD5State;
                      Count: MD5Count;
                      Buffer: MD5Buffer;
                     end;

Procedure MD5Init(var Context: MD5Context);
Procedure MD5Update(var Context: MD5Context; Input: PChar; Length: longword);
Procedure MD5Final(var Context: MD5Context; var Digest: MD5Digest);

Function MD5String(M: string): MD5Digest;
Function MD5File(N: string): MD5Digest;
Function MD5Print(D: MD5Digest): string;
Function MD5PrintFull(D: MD5Digest): String;
Function MD5Signature(D: MD5Digest): String;
Function MD5Match(D1, D2: MD5Digest): boolean;

IMPLEMENTATION

var PADDING: MD5Buffer = (
                $80, $00, $00, $00, $00, $00, $00, $00,
                $00, $00, $00, $00, $00, $00, $00, $00,
                $00, $00, $00, $00, $00, $00, $00, $00,
                $00, $00, $00, $00, $00, $00, $00, $00,
                $00, $00, $00, $00, $00, $00, $00, $00,
                $00, $00, $00, $00, $00, $00, $00, $00,
                $00, $00, $00, $00, $00, $00, $00, $00,
                $00, $00, $00, $00, $00, $00, $00, $00
                );

function F(x, y, z: DWORD): DWORD;
Begin
 Result := (x and y) or ((not x) and z);
end;

function G(x, y, z: DWORD): DWORD;
Begin
 Result := (x and z) or (y and (not z));
end;

function H(x, y, z: DWORD): DWORD;
Begin
 Result := x xor y xor z;
end;

function I(x, y, z: DWORD): DWORD;
Begin
 Result := y xor (x or (not z));
end;

Procedure rot(var x: DWORD; n: BYTE);
Begin
 x := (x shl n) or (x shr (32 - n));
end;

Procedure FF(var a: DWORD; b, c, d, x: DWORD; s: BYTE; ac: DWORD);
Begin
 Inc(a, F(b, c, d) + x + ac);
 rot(a, s);
 Inc(a, b);
end;

Procedure GG(var a: DWORD; b, c, d, x: DWORD; s: BYTE; ac: DWORD);
Begin
 Inc(a, G(b, c, d) + x + ac);
 rot(a, s);
 Inc(a, b);
end;

Procedure HH(var a: DWORD; b, c, d, x: DWORD; s: BYTE; ac: DWORD);
Begin
 Inc(a, H(b, c, d) + x + ac);
 rot(a, s);
 Inc(a, b);
end;

Procedure II(var a: DWORD; b, c, d, x: DWORD; s: BYTE; ac: DWORD);
Begin
 Inc(a, I(b, c, d) + x + ac);
 rot(a, s);
 Inc(a, b);
end;


Procedure Encode(Source, Target: pointer; Count: longword);
var
 S: PByte;
 T: PDWORD;
 I: longword;
Begin
 S := Source;
 T := Target;
 for I := 1 to Count div 4 do
  Begin
   T^ := S^;
   Inc(S);
   T^ := T^ or (S^ shl 8);
   Inc(S);
   T^ := T^ or (S^ shl 16);
   Inc(S);
   T^ := T^ or (S^ shl 24);
   Inc(S);
   Inc(T);
  end;
end;

// Decode Count DWORDs at Source into (Count * 4) Bytes at Target
Procedure Decode(Source, Target: pointer; Count: longword);
var
 S: PDWORD;
 T: PByte;
 I: longword;
Begin
 S := Source;
 T := Target;
 for I := 1 to Count do Begin
  T^ := S^ and $ff;
  Inc(T);
  T^ := (S^ shr 8) and $ff;
  Inc(T);
  T^ := (S^ shr 16) and $ff;
  Inc(T);
  T^ := (S^ shr 24) and $ff;
  Inc(T);
  Inc(S);
 end;
end;

// Transform State according to first 64 bytes at Buffer
Procedure Transform(Buffer: pointer; var State: MD5State);
var
 a, b, c, d: DWORD;
 Block: MD5Block;
Begin
 Encode(Buffer, @Block, 64);
 a := State[0];
 b := State[1];
 c := State[2];
 d := State[3];
 FF (a, b, c, d, Block[ 0],  7, $d76aa478);
 FF (d, a, b, c, Block[ 1], 12, $e8c7b756);
 FF (c, d, a, b, Block[ 2], 17, $242070db);
 FF (b, c, d, a, Block[ 3], 22, $c1bdceee);
 FF (a, b, c, d, Block[ 4],  7, $f57c0faf);
 FF (d, a, b, c, Block[ 5], 12, $4787c62a);
 FF (c, d, a, b, Block[ 6], 17, $a8304613);
 FF (b, c, d, a, Block[ 7], 22, $fd469501);
 FF (a, b, c, d, Block[ 8],  7, $698098d8);
 FF (d, a, b, c, Block[ 9], 12, $8b44f7af);
 FF (c, d, a, b, Block[10], 17, $ffff5bb1);
 FF (b, c, d, a, Block[11], 22, $895cd7be);
 FF (a, b, c, d, Block[12],  7, $6b901122);
 FF (d, a, b, c, Block[13], 12, $fd987193);
 FF (c, d, a, b, Block[14], 17, $a679438e);
 FF (b, c, d, a, Block[15], 22, $49b40821);
 GG (a, b, c, d, Block[ 1],  5, $f61e2562);
 GG (d, a, b, c, Block[ 6],  9, $c040b340);
 GG (c, d, a, b, Block[11], 14, $265e5a51);
 GG (b, c, d, a, Block[ 0], 20, $e9b6c7aa);
 GG (a, b, c, d, Block[ 5],  5, $d62f105d);
 GG (d, a, b, c, Block[10],  9,  $2441453);
 GG (c, d, a, b, Block[15], 14, $d8a1e681);
 GG (b, c, d, a, Block[ 4], 20, $e7d3fbc8);
 GG (a, b, c, d, Block[ 9],  5, $21e1cde6);
 GG (d, a, b, c, Block[14],  9, $c33707d6);
 GG (c, d, a, b, Block[ 3], 14, $f4d50d87);
 GG (b, c, d, a, Block[ 8], 20, $455a14ed);
 GG (a, b, c, d, Block[13],  5, $a9e3e905);
 GG (d, a, b, c, Block[ 2],  9, $fcefa3f8);
 GG (c, d, a, b, Block[ 7], 14, $676f02d9);
 GG (b, c, d, a, Block[12], 20, $8d2a4c8a);
 HH (a, b, c, d, Block[ 5],  4, $fffa3942);
 HH (d, a, b, c, Block[ 8], 11, $8771f681);
 HH (c, d, a, b, Block[11], 16, $6d9d6122);
 HH (b, c, d, a, Block[14], 23, $fde5380c);
 HH (a, b, c, d, Block[ 1],  4, $a4beea44);
 HH (d, a, b, c, Block[ 4], 11, $4bdecfa9);
 HH (c, d, a, b, Block[ 7], 16, $f6bb4b60);
 HH (b, c, d, a, Block[10], 23, $bebfbc70);
 HH (a, b, c, d, Block[13],  4, $289b7ec6);
 HH (d, a, b, c, Block[ 0], 11, $eaa127fa);
 HH (c, d, a, b, Block[ 3], 16, $d4ef3085);
 HH (b, c, d, a, Block[ 6], 23,  $4881d05);
 HH (a, b, c, d, Block[ 9],  4, $d9d4d039);
 HH (d, a, b, c, Block[12], 11, $e6db99e5);
 HH (c, d, a, b, Block[15], 16, $1fa27cf8);
 HH (b, c, d, a, Block[ 2], 23, $c4ac5665);
 II (a, b, c, d, Block[ 0],  6, $f4292244);
 II (d, a, b, c, Block[ 7], 10, $432aff97);
 II (c, d, a, b, Block[14], 15, $ab9423a7);
 II (b, c, d, a, Block[ 5], 21, $fc93a039);
 II (a, b, c, d, Block[12],  6, $655b59c3);
 II (d, a, b, c, Block[ 3], 10, $8f0ccc92);
 II (c, d, a, b, Block[10], 15, $ffeff47d);
 II (b, c, d, a, Block[ 1], 21, $85845dd1);
 II (a, b, c, d, Block[ 8],  6, $6fa87e4f);
 II (d, a, b, c, Block[15], 10, $fe2ce6e0);
 II (c, d, a, b, Block[ 6], 15, $a3014314);
 II (b, c, d, a, Block[13], 21, $4e0811a1);
 II (a, b, c, d, Block[ 4],  6, $f7537e82);
 II (d, a, b, c, Block[11], 10, $bd3af235);
 II (c, d, a, b, Block[ 2], 15, $2ad7d2bb);
 II (b, c, d, a, Block[ 9], 21, $eb86d391);
 Inc(State[0], a);
 Inc(State[1], b);
 Inc(State[2], c);
 Inc(State[3], d);
end;

Procedure MD5Init(var Context: MD5Context);
Begin
 With Context do
  Begin
   State[0] := $67452301;
   State[1] := $efcdab89;
   State[2] := $98badcfe;
   State[3] := $10325476;
   Count[0] := 0;
   Count[1] := 0;
   ZeroMemory(@Buffer, SizeOf(MD5Buffer));
  End;
End;

// Update given Context to include Length bytes of Input
Procedure MD5Update(var Context: MD5Context; Input: pChar; Length: longword);
Var
 Index  : longword;
 PartLen: longword;
 I      : longword;
Begin
 With Context do
  Begin
   Index := (Count[0] shr 3) and $3f;
   Inc(Count[0], Length shl 3);
   If Count[0] < (Length shl 3) then Inc(Count[1]);
   Inc(Count[1], Length shr 29);
  End;

 PartLen := 64 - Index;
 If Length >= PartLen then
  Begin
   CopyMemory(@Context.Buffer[Index], Input, PartLen);
   Transform(@Context.Buffer, Context.State);
   I := PartLen;
   While I + 63 < Length do
    Begin
     Transform(@Input[I], Context.State);
     Inc(I, 64);
    End;
   Index := 0;
   End
    Else
     I := 0;
 CopyMemory(@Context.Buffer[Index], @Input[I], Length - I);
End;

// Finalize given Context, create Digest and zeroize Context
Procedure MD5Final(var Context: MD5Context; var Digest: MD5Digest);
var
 Bits: MD5CBits;
 Index: longword;
 PadLen: longword;
Begin
 Decode(@Context.Count, @Bits, 2);
 Index := (Context.Count[0] shr 3) and $3f;
 if Index < 56 then PadLen := 56 - Index else PadLen := 120 - Index;
 MD5Update(Context, @PADDING, PadLen);
 MD5Update(Context, @Bits, 8);
 Decode(@Context.State, @Digest, 4);
 ZeroMemory(@Context, SizeOf(MD5Context));
end;

// -----------------------------------------------------------------------------------------------

// Create digest of given Message
function MD5String(M: string): MD5Digest;
var
 Context: MD5Context;
Begin
 MD5Init(Context);
 MD5Update(Context, pChar(M), length(M));
 MD5Final(Context, Result);
end;

// Create digest of file with given Name
function MD5File(N: string): MD5Digest;
var
 FileHandle: THandle;
 MapHandle: THandle;
 ViewPointer: pointer;
 Context: MD5Context;
Begin
 MD5Init(Context);
 FileHandle := CreateFile(pChar(N), GENERIC_READ, FILE_SHARE_READ or FILE_SHARE_WRITE,
     nil, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL or FILE_FLAG_SEQUENTIAL_SCAN, 0);
 If FileHandle <> INVALID_HANDLE_VALUE then
  Try
   MapHandle := CreateFileMapping(FileHandle, nil, PAGE_READONLY, 0, 0, nil);
   If MapHandle <> 0 then
    Try
     ViewPointer := MapViewOfFile(MapHandle, FILE_MAP_READ, 0, 0, 0);
     If ViewPointer <> nil then
      Try
       MD5Update(Context, ViewPointer, GetFileSize(FileHandle, nil));
      Finally
       UnmapViewOfFile(ViewPointer);
      End;
    Finally
     CloseHandle(MapHandle);
    End;
   Finally
    CloseHandle(FileHandle);
   End;
 MD5Final(Context, Result);
End;

// Create hex representation of given Digest
Function MD5Print(D: MD5Digest): string;

const  Digits: Array[0..15] Of Char =  ('0', '1', '2', '3', '4',
     '5', '6', '7', '8', '9',
     'a', 'b', 'c', 'd', 'e', 'f');

 var  {I: byte;}
      Block: Array[0..3] Of String;

Begin
 Result := '';

 Block[0]:= Digits[(D[00] shr 4) and $0f] + Digits[D[00] and $0f]+
            Digits[(D[01] shr 4) and $0f] + Digits[D[01] and $0f]+
            Digits[(D[02] shr 4) and $0f] + Digits[D[02] and $0f]+
            Digits[(D[03] shr 4) and $0f] + Digits[D[03] and $0f];

 Block[1]:= Digits[(D[04] shr 4) and $0f] + Digits[D[04] and $0f]+
            Digits[(D[05] shr 4) and $0f] + Digits[D[05] and $0f]+
            Digits[(D[06] shr 4) and $0f] + Digits[D[06] and $0f]+
            Digits[(D[07] shr 4) and $0f] + Digits[D[07] and $0f];

 Block[2]:= Digits[(D[08] shr 4) and $0f] + Digits[D[08] and $0f]+
            Digits[(D[09] shr 4) and $0f] + Digits[D[09] and $0f]+
            Digits[(D[10] shr 4) and $0f] + Digits[D[10] and $0f]+
            Digits[(D[11] shr 4) and $0f] + Digits[D[11] and $0f];

 Block[3]:= Digits[(D[12] shr 4) and $0f] + Digits[D[12] and $0f]+
            Digits[(D[13] shr 4) and $0f] + Digits[D[13] and $0f]+
            Digits[(D[14] shr 4) and $0f] + Digits[D[14] and $0f]+
            Digits[(D[15] shr 4) and $0f] + Digits[D[15] and $0f];

  Result:=Block[0]+' '+Block[1]+' '+Block[2]+' '+Block[3];

end;


// Create hex representation of given Digest
Function MD5PrintFull(D: MD5Digest): string;

const  Digits: Array[0..15] of char =
  ('0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'a', 'b', 'c', 'd', 'e', 'f');

 var  {I: byte;}
      Block: Array[0..3] Of String;

Begin
 Result := '';

 Block[0]:= Digits[(D[00] Shr 4) and $0f] + Digits[D[00] and $0f]+
            Digits[(D[01] Shr 4) and $0f] + Digits[D[01] and $0f]+
            Digits[(D[02] Shr 4) and $0f] + Digits[D[02] and $0f]+
            Digits[(D[03] Shr 4) and $0f] + Digits[D[03] and $0f];

 Block[1]:= Digits[(D[04] Shr 4) and $0f] + Digits[D[04] and $0f]+
            Digits[(D[05] Shr 4) and $0f] + Digits[D[05] and $0f]+
            Digits[(D[06] Shr 4) and $0f] + Digits[D[06] and $0f]+
            Digits[(D[07] Shr 4) and $0f] + Digits[D[07] and $0f];

 Block[2]:= Digits[(D[08] Shr 4) and $0f] + Digits[D[08] and $0f]+
            Digits[(D[09] Shr 4) and $0f] + Digits[D[09] and $0f]+
            Digits[(D[10] Shr 4) and $0f] + Digits[D[10] and $0f]+
            Digits[(D[11] Shr 4) and $0f] + Digits[D[11] and $0f];

 Block[3]:= Digits[(D[12] Shr 4) and $0f] + Digits[D[12] and $0f]+
            Digits[(D[13] Shr 4) and $0f] + Digits[D[13] and $0f]+
            Digits[(D[14] Shr 4) and $0f] + Digits[D[14] and $0f]+
            Digits[(D[15] Shr 4) and $0f] + Digits[D[15] and $0f];

 Result:=Block[0]+Block[1]+Block[2]+Block[3];

End;


Function MD5Signature(D: MD5Digest): String;
Var Str:String;
    I  :Integer;
Begin
 For I:= 0 to 15 Do
  Begin
   Str:= Str + Char(D[I] Mod 255);
  End;
 Result:= Str;
End;


Function MD5Match(D1, D2: MD5Digest): boolean;
Var
 I: byte;
Begin
 I := 0;
 Result := TRUE;
 While Result And (I < 16) do
  Begin
   Result := D1[I] = D2[I];
   Inc(I);
  End;
End;

End.

