(*************************************************************************)
(*                                                                       *)
(*                                                                       *)
(*                         Watermark Library                             *)
(*                    Release Ver 0.0.1  26/06/2000                      *)
(*                                                                       *)
(* (C)opyright by Arash Partow                                           *)
(* You may use this library in conjunction with                          *)
(* the current version of CPL.                                           *)
(*                                                                       *)
(*                                                                       *)
(*************************************************************************)

Unit WMImageUnit;

Interface

Uses  SysUtils,Types,Graphics, Dialogs, MD5;


Type TRGBTriple = Record
                   rgbtBlue : Byte;
                   rgbtGreen: Byte;
                   rgbtRed  : Byte;
                  end;


Const

      BIT0ON       = $01; {00000001}
      BIT1ON       = $02; {00000010}
      BIT2ON       = $04; {00000100}
      BIT3ON       = $08; {00001000}
      BIT4ON       = $10; {00010000}
      BIT5ON       = $20; {00100000}
      BIT6ON       = $30; {01000000}
      BIT7ON       = $40; {10000000}
      ALLBITSON    = $FF; {11111111}


      BIT0OFF      = $FE; {11111110}
      BIT1OFF      = $FD; {11111101}
      BIT2OFF      = $FB; {11111011}
      BIT3OFF      = $F7; {11110111}
      BIT4OFF      = $EF; {11101111}
      BIT5OFF      = $DF; {11011111}
      BIT6OFF      = $BF; {10111111}
      BIT7OFF      = $7F; {01111111}
      ALLBITSOFF   = $00; {00000000}

      
      LSBOFF       = $FE;
      LSBMASK      = $01;

      RedChannelPixel   : TRGBTriple = (rgbtBlue:$00; rgbtGreen:$00; rgbtRed:$FF);
      GreenChannelPixel : TRGBTriple = (rgbtBlue:$00; rgbtGreen:$FF; rgbtRed:$00);
      BlueChannelPixel  : TRGBTriple = (rgbtBlue:$FF; rgbtGreen:$00; rgbtRed:$00);
      BlackPixel        : TRGBTriple = (rgbtBlue:$00; rgbtGreen:$00; rgbtRed:$00);

Type RGBTripleArrayPtr = ^TRGBTripleArray;
     TRGBTripleArray   = Array[Word] Of TRGBTriple;


Type WaterMarkImage = Class


      Private

       OriginalImage:TBitMap;
       RCWM         :TBitMap;
       GCWM         :TBitMap;
       BCWM         :TBitMap;

       OIL          :Boolean;
       RWML         :Boolean;
       GWML         :Boolean;
       BWML         :Boolean;



      Public


       Constructor Create;

       Procedure SetOriginalImage(Image:TBitMap);
       Procedure SetRedChannelWM(Image:TBitMap);
       Procedure SetGreenChannelWM(Image:TBitMap);
       Procedure SetBlueChannelWM(Image:TBitMap);

       Function GetOriginalImage:TBitMap;
       Function GetRedChannelWM:TBitMap;
       Function GetGreenChannelWM:TBitMap;
       Function GetBlueChannelWM:TBitMap;

       Procedure CreateWaterMarkTemplate(WMImage:TBitMap; Var BitMap:TBitMap);


       Procedure ExtractSimpleRedChannel(Var BitMap:TBitMap);
       Procedure ExtractSimpleGreenChannel(Var BitMap:TBitMap);
       Procedure ExtractSimpleBlueChannel(Var BitMap:TBitMap);
       Procedure ExtractSimpleKeyRedChannel(KeyStr:String; Var BitMap:TBitMap);
       Procedure ExtractSimpleKeyGreenChannel(KeyStr:String; Var BitMap:TBitMap);
       Procedure ExtractSimpleKeyBlueChannel(KeyStr:String; Var BitMap:TBitMap);

       Function ApplySimpleRedChannel:TBitMap;
       Function ApplySimpleGreenChannel:TBitMap;
       Function ApplySimpleBlueChannel:TBitMap;
       Function ApplySimpleAllChannel:TBitMap;

       Function ApplySimpleKeyRedChannel(KeyStr:String):TBitMap;
       Function ApplySimpleKeyGreenChannel(KeyStr:String):TBitMap;
       Function ApplySimpleKeyBlueChannel(KeyStr:String):TBitMap;
       Function ApplySimpleKeyAllChannel(KeyStrRC,KeyStrGC,KeyStrBC:String):TBitMap;

       Function ApplySimpleMD5RedChannel:TBitMap;
       Function ApplySimpleMD5GreenChannel:TBitMap;
       Function ApplySimpleMD5BlueChannel:TBitMap;
       Function ApplySimpleMD5AllChannel:TBitMap;


       Function PaddWaterMarkKey(Key:String):String;
       Function HashWaterMarkKey(Key:String):String;


       Procedure TurnOffBit(Bit:Byte; Var BitMap:TBitMap);

       
       Property  OIExists  : Boolean read OIL;
       Property  RWMExists : Boolean read RWML;
       Property  GWMExists : Boolean read GWML;
       Property  BWMExists : Boolean read BWML;



     End;


Type WMInfo = Record
      Name         : String;
      CompanyName  : String;
      Copyright    : String;
      TimeStamp    : String;
      SecureKey    : String;
      FileName     : String;
      RandBkGround : Boolean;
      InvBkGround  : Boolean;
     End;


Implementation

Constructor WaterMarkImage.Create;
Begin
 OIL  := False;
 RWML := False;
 GWML := False;
 BWML := False;
End;

Procedure WaterMarkImage.SetOriginalImage(Image:TBitMap);
Begin
 OriginalImage             := TBitMap.Create;
 OriginalImage.PixelFormat := Image.PixelFormat;
 OriginalImage.Width       := Image.Width;
 OriginalImage.Height      := Image.Height;
 OriginalImage.Assign(Image);
 OIL:=True;
End;

Procedure WaterMarkImage.SetRedChannelWM(Image:TBitMap);
Begin
 RCWM             := TBitMap.Create;
 RCWM.PixelFormat := pf1bit;
 RCWM.Width       := Image.Width;
 RCWM.Height      := Image.Height;
 RCWM.Assign(Image);
 RWML := True;
End;

Procedure WaterMarkImage.SetGreenChannelWM(Image:TBitMap);
Begin
 GCWM             := TBitMap.Create;
 GCWM.PixelFormat := pf1bit;
 GCWM.Width       := Image.Width;
 GCWM.Height      := Image.Height;
 GCWM.Assign(Image);
 GWML := True;
End;

Procedure WaterMarkImage.SetBlueChannelWM(Image:TBitMap);
Begin
 BCWM             := TBitMap.Create;
 BCWM.PixelFormat := pf1bit;
 BCWM.Width       := Image.Width;
 BCWM.Height      := Image.Height;
 BCWM.Assign(Image);
 BWML := True;
End;


Function WaterMarkImage.GetOriginalImage:TBitMap;
Begin
 Result:= OriginalImage;
End;

Function WaterMarkImage.GetRedChannelWM:TBitMap;
Begin
 Result:= RCWM;
End;

Function WaterMarkImage.GetGreenChannelWM:TBitMap;
Begin
 Result:= GCWM;
End;

Function WaterMarkImage.GetBlueChannelWM:TBitMap;
Begin
 Result:= BCWM;
End;


Procedure WaterMarkImage.CreateWaterMarkTemplate(WMImage:TBitMap; Var BitMap:TBitMap);
Var
 NumAcross: Integer;
 NumDown  : Integer;
 x,y      : Integer;
Begin
 BitMap             := TBitMap.Create;
 Bitmap.PixelFormat := pf1Bit;
 Bitmap.Width       := OriginalImage.Width;
 Bitmap.Height      := OriginalImage.Height;

 NumAcross := Bitmap.Width Div WMImage.Width;
 If (Bitmap.Width mod WMImage.Width) > 0 Then Inc(numAcross);

 NumDown := Bitmap.Height Div WMImage.Height;
 If (Bitmap.Height mod WMImage.Height) > 0 Then Inc(numDown);

 For y :=0 To NumDown-1 Do
  Begin
   For x:=0 To NumAcross-1 Do
    Begin
     BitMap.Canvas.Draw(x*WMImage.Width, y*WMImage.Height,WMImage);
    End;
  End;
 Bitmap.Width       := OriginalImage.Width;
 Bitmap.Height      := OriginalImage.Height;
 Bitmap.PixelFormat := pf24Bit;
End;


Procedure WaterMarkImage.ExtractSimpleRedChannel(Var BitMap:TBitMap);
Var
 x,y    : Integer;
 Pixel  : RGBTripleArrayPtr;
Begin

 If Not Assigned(OriginalImage) Then
  Begin
   Exit;
  End;
 If Assigned(BitMap) Then BitMap.Free;
 BitMap             := TBitMap.Create;
 BitMap.Assign(OriginalImage);

 For y := 0 to Pred(BitMap.Height) Do
  Begin
   Pixel := BitMap.ScanLine[y];
    For x := 0 To Pred(BitMap.Width) Do
     Begin
      If (Pixel[x].rgbtRed And 1) = 0 Then Pixel[x]:=BlackPixel
       Else
        Pixel[x] :=RedChannelPixel ;
     End;
  End;

End;


Procedure WaterMarkImage.ExtractSimpleGreenChannel(Var BitMap:TBitMap);
Var
 x,y    : Integer;
 Pixel  : RGBTripleArrayPtr;
Begin

 If Not Assigned(OriginalImage) Then
  Begin
   Exit;
  End;
 If Assigned(BitMap) Then BitMap.Free;
 BitMap             := TBitMap.Create;
 BitMap.Assign(OriginalImage);

  For y := 0 to Pred(BitMap.Height) Do
   Begin
    Pixel := BitMap.ScanLine[y];
    For x := 0 To Pred(BitMap.Width) Do
     Begin
      If (Pixel[x].rgbtGreen And 1) = 0 Then Pixel[x]:=BlackPixel
       Else
        Pixel[x] :=GreenChannelPixel;
     End;
   End;

End;


Procedure WaterMarkImage.ExtractSimpleBlueChannel(Var BitMap:TBitMap);
Var
 x,y    : Integer;
 Pixel  : RGBTripleArrayPtr;
Begin

 If Not Assigned(OriginalImage) Then
  Begin
   Exit;
  End;
 If Assigned(BitMap) Then BitMap.Free;
 BitMap             := TBitMap.Create;
 BitMap.Assign(OriginalImage);

 For y := 0 to Pred(BitMap.Height) Do
  Begin
   Pixel := BitMap.ScanLine[y];
   For x := 0 To Pred(BitMap.Width) Do
    Begin
     If (Pixel[x].rgbtRed And 1) = 0 Then Pixel[x]:=BlackPixel
      Else
       Pixel[x] :=BlueChannelPixel;
    End;
  End;

End;


Procedure WaterMarkImage.ExtractSimpleKeyRedChannel(KeyStr:String; Var BitMap:TBitMap);
Var
 x,y,z     : Integer;
 PixelI    : Array[0..7] Of RGBTripleArrayPtr;
 PixelW    : Array[0..7] Of RGBTripleArrayPtr;
 WBMP      : TBitMap;
 KeyBitPos : Byte;
 KeyBit    : Byte;
 Key       : Array[0..15] Of Byte;
 Across    : Integer;
 Down      : Integer;
 AcCount   : Integer;
 DnCount   : Integer;
Begin

 KeyStr:=HashWaterMarkKey(KeyStr);
 For Z:= 0 to 15 do Key[Z]:=Ord(KeyStr[Z+1]);

 If Not Assigned(OriginalImage) Then
  Begin
   Exit;
  End;

 WBMP             := TBitMap.Create;
 WBMP.PixelFormat := pf24Bit;
 WBMP.Width       := OriginalImage.Width  + (OriginalImage.Width  mod 16);
 WBMP.Height      := OriginalImage.Height + (OriginalImage.Height mod 8);
 WBMP.Canvas.Brush.Color:=clBlack;
 WBMP.Canvas.FillRect(Rect(0,0,OriginalImage.Width,OriginalImage.Height));

 Across:= OriginalImage.Width  Div 16;
 Down  := OriginalImage.Height Div  8;

 For DnCount := 0 to Down-1 do
  Begin

   For z:= 0 to 7 Do
    Begin
     PixelI[z]:= OriginalImage.ScanLine[(DnCount Shl 3)+z];
     PixelW[z]:= WBMP.ScanLine[(DnCount Shl 3)+z];
    End;

   For y:= 0 to 7 do
    Begin

     For x := 0 to 15 do
      Begin
       KeyBitPos:=(y Shl 4)+x;
       KeyBit:=((Key[(KeyBitPos Shr 3)] Shr (7-(KeyBitPos And 7))) And $01);
       For AcCount:= 0 To Across-1 Do
        If ((PixelI[y][x+(AcCount Shl 4)].rgbtRed And $01) Xor KeyBit) = 1 then
         Begin
          PixelW[y][x+(AcCount Shl 4)].rgbtRed :=$FF;
         End;
      End;

    End;

  End;

 BitMap             := TBitMap.Create;
 BitMap.PixelFormat := pf24Bit;
 BitMap.Width       := OriginalImage.Width;
 BitMap.Height      := OriginalImage.Height;
 BitMap.Canvas.Draw(0,0,WBMP);
 WBMP.Free;

End;


Procedure WaterMarkImage.ExtractSimpleKeyGreenChannel(KeyStr:String; Var BitMap:TBitMap);
Var
 x,y,z     : Integer;
 PixelI    : Array[0..7] Of RGBTripleArrayPtr;
 PixelW    : Array[0..7] Of RGBTripleArrayPtr;
 WBMP      : TBitMap;
 KeyBitPos : Byte;
 KeyBit    : Byte;
 Key       : Array[0..15] Of Byte;
 Across    : Integer;
 Down      : Integer;
 AcCount   : Integer;
 DnCount   : Integer;
Begin

 KeyStr:=HashWaterMarkKey(KeyStr);
 For Z:= 0 to 15 do Key[Z]:=Ord(KeyStr[Z+1]);

 WBMP             := TBitMap.Create;
 WBMP.PixelFormat := pf24Bit;
 WBMP.Width       := OriginalImage.Width  + (OriginalImage.Width  mod 16);
 WBMP.Height      := OriginalImage.Height + (OriginalImage.Height mod 8);
 WBMP.Canvas.Brush.Color:=clBlack;
 WBMP.Canvas.FillRect(Rect(0,0,OriginalImage.Width,OriginalImage.Height));

 Across:= OriginalImage.Width  Div 16;
 Down  := OriginalImage.Height Div  8;

 For DnCount := 0 to Down-1 do
  Begin
   For z:= 0 to 7 Do
    Begin
     PixelI[z]:= OriginalImage.ScanLine[(DnCount Shl 3)+z];
     PixelW[z]:= WBMP.ScanLine[(DnCount Shl 3)+z];
    End;
     For y:= 0 to 7 do
      Begin
       For x := 0 to 15 do
        Begin
         KeyBitPos:=(y Shl 4)+x;
         KeyBit:=((Key[(KeyBitPos Shr 3)] Shr (7-(KeyBitPos And 7))) And $01);
          For AcCount:= 0 To Across-1 Do
            If ((PixelI[y][x+(AcCount Shl 4)].rgbtGreen And $01) Xor KeyBit) = 1 then
             Begin
              PixelW[y][x+(AcCount Shl 4)].rgbtGreen :=$FF;
             End;
        End;
      End;
  End;

 BitMap             := TBitMap.Create;
 BitMap.PixelFormat := pf24Bit;
 BitMap.Width       := OriginalImage.Width;
 BitMap.Height      := OriginalImage.Height;
 BitMap.Canvas.Draw(0,0,WBMP);
 WBMP.Free;

End;

Procedure WaterMarkImage.ExtractSimpleKeyBlueChannel(KeyStr:String; Var BitMap:TBitMap);
Var
 x,y,z     : Integer;
 PixelI    : Array[0..7] Of RGBTripleArrayPtr;
 PixelW    : Array[0..7] Of RGBTripleArrayPtr;
 WBMP      : TBitMap;
 KeyBitPos : Byte;
 KeyBit    : Byte;
 Key       : Array[0..15] Of Byte;
 Across    : Integer;
 Down      : Integer;
 AcCount   : Integer;
 DnCount   : Integer;
Begin

 KeyStr:=HashWaterMarkKey(KeyStr);
 For Z:= 0 to 15 do Key[Z]:=Ord(KeyStr[Z+1]);

 If Not Assigned(OriginalImage) Then Exit;
 WBMP             := TBitMap.Create;
 WBMP.PixelFormat := pf24Bit;
 WBMP.Width       := OriginalImage.Width  + (OriginalImage.Width  mod 16);
 WBMP.Height      := OriginalImage.Height + (OriginalImage.Height mod 8);
 WBMP.Canvas.Brush.Color:=clBlack;
 WBMP.Canvas.FillRect(Rect(0,0,OriginalImage.Width,OriginalImage.Height));

 Across:= OriginalImage.Width  Div 16;
 Down  := OriginalImage.Height Div  8;

 For DnCount := 0 to Down-1 do
  Begin
   For z:= 0 to 7 Do
    Begin
     PixelI[z]:= OriginalImage.ScanLine[(DnCount Shl 3)+z];
     PixelW[z]:= WBMP.ScanLine[(DnCount Shl 3)+z];
    End;
     For y:= 0 to 7 do
      Begin
       For x := 0 to 15 do
        Begin
         KeyBitPos:=(y Shl 4)+x;
         KeyBit:=((Key[(KeyBitPos Shr 3)] Shr (7-(KeyBitPos And 7))) And $01);
          For AcCount:= 0 To Across-1 Do
            If ((PixelI[y][x+(AcCount Shl 4)].rgbtBlue And $01) Xor KeyBit) = 1 then
             Begin
              PixelW[y][x+(AcCount Shl 4)].rgbtBlue :=$FF;
             End;
        End;
      End;
  End;
 BitMap             := TBitMap.Create;
 BitMap.PixelFormat := pf24Bit;
 BitMap.Width       := OriginalImage.Width;
 BitMap.Height      := OriginalImage.Height;
 BitMap.Canvas.Draw(0,0,WBMP);
 WBMP.Free;

End;


Function WaterMarkImage.ApplySimpleRedChannel:TBitMap;
Var
 x,y       : Integer;
 PixelI    : RGBTripleArrayPtr;
 PixelT    : RGBTripleArrayPtr;
 PixelW    : RGBTripleArrayPtr;
 BitMap    : TBitMap;
 WMTemplate: TBitMap;
Begin
 Result := Nil;
 If Not Assigned(OriginalImage) Then Exit;

 BitMap             := TBitMap.Create;
 Bitmap.PixelFormat := pf24Bit;
 Bitmap.Width       := OriginalImage.Width;
 Bitmap.Height      := OriginalImage.Height;
 BitMap.Canvas.Draw(0,0,OriginalImage);

 If Not Assigned (RCWM) Then
  Begin
   Result:=BitMap;
   MessageDlg('Red Watermark has not been loaded!'+#13+'Cannot continue with current operation.', mtError      ,[mbOk], 0);
   Exit;
  End;

 CreateWaterMarkTemplate(RCWM,WMTemplate);


 For y := 0 to OriginalImage.Height-1 do
  Begin
   PixelI := OriginalImage.ScanLine[y];
   PixelT := WMTemplate.ScanLine[y];
   PixelW := BitMap.ScanLine[y];
   For x := 0 to OriginalImage.Width-1 do
    Begin
     PixelW[x].rgbtRed:= (PixelI[x].rgbtRed And LSBOFF) Or
                         (PixelT[x].rgbtRed And LSBMASK);
    End;
  End;

 Result:=BitMap;

End;


Function WaterMarkImage.ApplySimpleGreenChannel:TBitMap;
Var
 x,y       : Integer;
 PixelI    : RGBTripleArrayPtr;
 PixelT    : RGBTripleArrayPtr;
 PixelW    : RGBTripleArrayPtr;
 BitMap    : TBitMap;
 WMTemplate: TBitMap;
Begin
 Result := Nil;
 If Not Assigned(OriginalImage) Then Exit;

 BitMap             := TBitMap.Create;
 Bitmap.PixelFormat := pf24Bit;
 Bitmap.Width       := OriginalImage.Width;
 Bitmap.Height      := OriginalImage.Height;
 BitMap.Canvas.Draw(0,0,OriginalImage);

 If Not Assigned (GCWM) Then
  Begin
   Result:=BitMap;
   MessageDlg('Green Watermark has not been loaded!'+#13+'Cannot continue with current operation.', mtError      ,[mbOk], 0);
   Exit;
  End;

 CreateWaterMarkTemplate(GCWM,WMTemplate);
 For y := 0 to OriginalImage.Height-1 do
  Begin
   PixelI := OriginalImage.ScanLine[y];
   PixelT := WMTemplate.ScanLine[y];
   PixelW := BitMap.ScanLine[y];
   For x := 0 to OriginalImage.Width-1 do
    Begin
     PixelW[x].rgbtGreen:= (PixelI[x].rgbtGreen And LSBOFF) Or
                         (PixelT[x].rgbtGreen And LSBMASK);
    End;
  End;
 Result:=BitMap;
End;

Function WaterMarkImage.ApplySimpleBlueChannel:TBitMap;
Var  x,y       : Integer;
     PixelI    : RGBTripleArrayPtr;
     PixelT    : RGBTripleArrayPtr;
     PixelW    : RGBTripleArrayPtr;
     BitMap    : TBitMap;
     WMTemplate: TBitMap;
Begin

 Result := Nil;

 If Not Assigned(OriginalImage) Then Exit;

 BitMap             := TBitMap.Create;
 Bitmap.PixelFormat := pf24Bit;
 Bitmap.Width       := OriginalImage.Width;
 Bitmap.Height      := OriginalImage.Height;
 BitMap.Canvas.Draw(0,0,OriginalImage);


 If Not Assigned (BCWM) Then
  Begin
   Result:=BitMap;
   MessageDlg('Blue Watermark has not been loaded!'+#13+'Cannot continue with current operation.', mtError      ,[mbOk], 0);
   Exit;
  End;


 CreateWaterMarkTemplate(BCWM,WMTemplate);


 For y := 0 to OriginalImage.Height-1 do
  Begin
   PixelI := OriginalImage.ScanLine[y];
   PixelT := WMTemplate.ScanLine[y];
   PixelW := BitMap.ScanLine[y];
   For x := 0 to OriginalImage.Width-1 do
    Begin
     PixelW[x].rgbtBlue:= (PixelI[x].rgbtBlue  And LSBOFF) Or
                          (PixelT[x].rgbtBlue And LSBMASK);
    End;
  End;

 Result:=BitMap;

End;


Function WaterMarkImage.ApplySimpleAllChannel:TBitMap;
Var  x,y         : Integer;
     PixelI      : RGBTripleArrayPtr;
     PixelTR     : RGBTripleArrayPtr;
     PixelTG     : RGBTripleArrayPtr;
     PixelTB     : RGBTripleArrayPtr;
     PixelW      : RGBTripleArrayPtr;
     BitMap      : TBitMap;
     WMTemplateR : TBitMap;
     WMTemplateG : TBitMap;
     WMTemplateB : TBitMap;
Begin

 Result := Nil;

 If Not Assigned(OriginalImage) Then Exit;
 BitMap             := TBitMap.Create;
 Bitmap.PixelFormat := pf24Bit;
 Bitmap.Width       := OriginalImage.Width;
 Bitmap.Height      := OriginalImage.Height;
 BitMap.Canvas.Draw(0,0,OriginalImage);

 CreateWaterMarkTemplate(RCWM,WMTemplateR);
 CreateWaterMarkTemplate(GCWM,WMTemplateG);
 CreateWaterMarkTemplate(BCWM,WMTemplateB);


 For y := 0 to OriginalImage.Height-1 do
  Begin
   PixelI  := OriginalImage.ScanLine[y];
   PixelTR := WMTemplateR.ScanLine[y];
   PixelTG := WMTemplateG.ScanLine[y];
   PixelTB := WMTemplateB.ScanLine[y];
   PixelW  := BitMap.ScanLine[y];
   For x := 0 to OriginalImage.Width-1 do
    Begin

     PixelW[x].rgbtRed:= (PixelI[x].rgbtRed  And LSBOFF) Or
                         (PixelTR[x].rgbtRed And LSBMASK);

     PixelW[x].rgbtGreen:= (PixelI[x].rgbtGreen  And LSBOFF) Or
                           (PixelTG[x].rgbtGreen And LSBMASK);

     PixelW[x].rgbtBlue:= (PixelI[x].rgbtBlue  And LSBOFF) Or
                          (PixelTB[x].rgbtBlue And LSBMASK);

    End;
  End;

 Result:=BitMap;

End;


Function WaterMarkImage.ApplySimpleKeyRedChannel(KeyStr:String):TBitMap;
Var
 x,y,z       : Integer;
 PixelI      : Array[0..7] Of RGBTripleArrayPtr;
 PixelT      : Array[0..7] Of RGBTripleArrayPtr;
 OBMP        : TBitMap;
 WMTemplate  : TBitMap;
 WBMP        : TBitMap;
 BitMap      : TBitMap;
 KeyBitPos   : Byte;
 KeyBit      : Byte;
 Key         : Array[0..15] Of Byte;
 Across      : Integer;
 Down        : Integer;
 AcCount     : Integer;
 DnCount     : Integer;
Begin

 Result := Nil;

 KeyStr:=HashWaterMarkKey(KeyStr);
 For Z:= 0 to 15 do Key[Z]:=Ord(KeyStr[Z+1]);

 OBMP             := TBitMap.Create;
 OBMP.PixelFormat := pf24Bit;
 OBMP.Width       := OriginalImage.Width  + (OriginalImage.Width  mod 16);
 OBMP.Height      := OriginalImage.Height + (OriginalImage.Height mod 8);
 OBMP.Canvas.Brush.Color:=clBlack;
 OBMP.Canvas.FillRect(Rect(0,0,OBMP.Width,OBMP.Height));
 OBMP.Canvas.Draw(0,0,OriginalImage);

 CreateWaterMarkTemplate(RCWM,WMTemplate);

 WBMP             := TBitMap.Create;
 WBMP.PixelFormat := pf24Bit;
 WBMP.Width       := OriginalImage.Width  + (OriginalImage.Width  mod 16);
 WBMP.Height      := OriginalImage.Height + (OriginalImage.Height mod 8);
 WBMP.Canvas.Brush.Color:=clBlack;
 WBMP.Canvas.FillRect(Rect(0,0,WBMP.Width,WBMP.Height));
 WBMP.Canvas.Draw(0,0,WMTemplate);
 WMTemplate.Destroy;

 Across:= OBMP.Width  Div 16;
 Down  := OBMP.Height Div  8;

 For DnCount := 0 to Down-1 do
  Begin

   For z:= 0 to 7 Do
    Begin
     PixelI[z]:= OBMP.ScanLine[(DnCount Shl 3)+z];
     PixelT[z]:= WBMP.ScanLine[(DnCount Shl 3)+z];
    End;
     For y:= 0 to 7 do
      Begin
       For x := 0 to 15 do
        Begin

         KeyBitPos:=(y Shl 4)+x;
         KeyBit:=((key[(KeyBitPos Shr 3)] Shr (7-(KeyBitPos And 7))) And $01);
          For AcCount:= 0 To Across-1 Do
           Begin

            PixelI[y][x+(AcCount Shl 4)].rgbtRed:= (PixelI[y][x+(AcCount Shl 4)].rgbtRed And $FE) Or
                                                  ((PixelT[y][x+(AcCount Shl 4)].rgbtRed Xor KeyBit) And $01);
           End;

        End;
    End;
  End;

 BitMap             := TBitMap.Create;
 BitMap.PixelFormat := pf24Bit;
 BitMap.Width       := OriginalImage.Width;
 BitMap.Height      := OriginalImage.Height;
 BitMap.Canvas.Draw(0,0,OBMP);
 BitMap.Width       := OriginalImage.Width;
 BitMap.Height      := OriginalImage.Height;
 Result := BitMap;

End;


Function WaterMarkImage.ApplySimpleKeyGreenChannel(KeyStr:String):TBitMap;
Var  x,y,z       : Integer;
     PixelI      : Array[0..7] Of RGBTripleArrayPtr;
     PixelT      : Array[0..7] Of RGBTripleArrayPtr;
     OBMP        : TBitMap;
     WMTemplate  : TBitMap;
     WBMP        : TBitMap;
     BitMap      : TBitMap;
     KeyBitPos   : Byte;
     KeyBit      : Byte;
     Key         : Array[0..15] Of Byte;
     Across      : Integer;
     Down        : Integer;
     AcCount     : Integer;
     DnCount     : Integer;
Begin

 Result := Nil;

 KeyStr:=HashWaterMarkKey(KeyStr);
 For Z:= 0 to 15 do Key[Z]:=Ord(KeyStr[Z+1]);

 OBMP             := TBitMap.Create;
 OBMP.PixelFormat := pf24Bit;
 OBMP.Width       := OriginalImage.Width  + (OriginalImage.Width  mod 16);
 OBMP.Height      := OriginalImage.Height + (OriginalImage.Height mod 8);
 OBMP.Canvas.Brush.Color:=clBlack;
 OBMP.Canvas.FillRect(Rect(0,0,OBMP.Width,OBMP.Height));
 OBMP.Canvas.Draw(0,0,OriginalImage);

 CreateWaterMarkTemplate(GCWM,WMTemplate);

 WBMP             := TBitMap.Create;
 WBMP.PixelFormat := pf24Bit;
 WBMP.Width       := OriginalImage.Width  + (OriginalImage.Width  mod 16);
 WBMP.Height      := OriginalImage.Height + (OriginalImage.Height mod 8);
 WBMP.Canvas.Brush.Color:=clBlack;
 WBMP.Canvas.FillRect(Rect(0,0,WBMP.Width,WBMP.Height));
 WBMP.Canvas.Draw(0,0,WMTemplate);
 WMTemplate.Destroy;

 Across:= OBMP.Width  Div 16;
 Down  := OBMP.Height Div  8;

 For DnCount := 0 to Down-1 do
  Begin

   For z:= 0 to 7 Do
    Begin
     PixelI[z]:= OBMP.ScanLine[(DnCount Shl 3)+z];
     PixelT[z]:= WBMP.ScanLine[(DnCount Shl 3)+z];
    End;
     For y:= 0 to 7 do
      Begin
       For x := 0 to 15 do
        Begin

         KeyBitPos:=(y Shl 4)+x;
         KeyBit:=((key[(KeyBitPos Shr 3)] Shr (7-(KeyBitPos And 7))) And $01);
          For AcCount:= 0 To Across-1 Do
           Begin

            PixelI[y][x+(AcCount Shl 4)].rgbtGreen:= (PixelI[y][x+(AcCount Shl 4)].rgbtGreen And $FE) Or
                                                    ((PixelT[y][x+(AcCount Shl 4)].rgbtGreen Xor KeyBit) And $01);
           End;

        End;
    End;
  End;

 BitMap             := TBitMap.Create;
 BitMap.PixelFormat := pf24Bit;
 BitMap.Width       := OriginalImage.Width;
 BitMap.Height      := OriginalImage.Height;
 BitMap.Canvas.Draw(0,0,OBMP);
 BitMap.Width       := OriginalImage.Width;
 BitMap.Height      := OriginalImage.Height;
 Result := BitMap;

End;


Function WaterMarkImage.ApplySimpleKeyBlueChannel(KeyStr:String):TBitMap;
Var  x,y,z       : Integer;
     PixelI      : Array[0..7] Of RGBTripleArrayPtr;
     PixelT      : Array[0..7] Of RGBTripleArrayPtr;
     OBMP        : TBitMap;
     WMTemplate  : TBitMap;
     WBMP        : TBitMap;
     BitMap      : TBitMap;
     KeyBitPos   : Byte;
     KeyBit      : Byte;
     Key         : Array[0..15] Of Byte;
     Across      : Integer;
     Down        : Integer;
     AcCount     : Integer;
     DnCount     : Integer;
Begin

 Result := Nil;

 KeyStr:=HashWaterMarkKey(KeyStr);
 For Z:= 0 to 15 do Key[Z]:=Ord(KeyStr[Z+1]);

 OBMP             := TBitMap.Create;
 OBMP.PixelFormat := pf24Bit;
 OBMP.Width       := OriginalImage.Width  + (OriginalImage.Width  mod 16);
 OBMP.Height      := OriginalImage.Height + (OriginalImage.Height mod 8);
 OBMP.Canvas.Brush.Color:=clBlack;
 OBMP.Canvas.FillRect(Rect(0,0,OBMP.Width,OBMP.Height));
 OBMP.Canvas.Draw(0,0,OriginalImage);

 CreateWaterMarkTemplate(BCWM,WMTemplate);

 WBMP             := TBitMap.Create;
 WBMP.PixelFormat := pf24Bit;
 WBMP.Width       := OriginalImage.Width  + (OriginalImage.Width  mod 16);
 WBMP.Height      := OriginalImage.Height + (OriginalImage.Height mod 8);
 WBMP.Canvas.Brush.Color:=clBlack;
 WBMP.Canvas.FillRect(Rect(0,0,WBMP.Width,WBMP.Height));
 WBMP.Canvas.Draw(0,0,WMTemplate);
 WMTemplate.Destroy;

 Across:= OBMP.Width  Div 16;
 Down  := OBMP.Height Div  8;

 For DnCount := 0 to Down-1 do
  Begin

   For z:= 0 to 7 Do
    Begin
     PixelI[z]:= OBMP.ScanLine[(DnCount Shl 3)+z];
     PixelT[z]:= WBMP.ScanLine[(DnCount Shl 3)+z];
    End;
     For y:= 0 to 7 do
      Begin
       For x := 0 to 15 do
        Begin

         KeyBitPos:=(y Shl 4)+x;
         KeyBit:=((key[(KeyBitPos Shr 3)] Shr (7-(KeyBitPos And 7))) And $01);
          For AcCount:= 0 To Across-1 Do
           Begin

            PixelI[y][x+(AcCount Shl 4)].rgbtBlue:=
             (PixelI[y][x+(AcCount Shl 4)].rgbtBlue And $FE) Or
            ((PixelT[y][x+(AcCount Shl 4)].rgbtBlue Xor KeyBit) And $01);

           End;

        End;
    End;
  End;

 BitMap             := TBitMap.Create;
 BitMap.PixelFormat := pf24Bit;
 BitMap.Width       := OriginalImage.Width;
 BitMap.Height      := OriginalImage.Height;
 BitMap.Canvas.Draw(0,0,OBMP);
 BitMap.Width       := OriginalImage.Width;
 BitMap.Height      := OriginalImage.Height;
 Result := BitMap;

End;


Function WaterMarkImage.ApplySimpleKeyAllChannel(KeyStrRC,KeyStrGC,KeyStrBC:String):TBitMap;
Var  x,y,z       : Integer;
     PixelI      : Array[0..7] Of RGBTripleArrayPtr;
     PixelTR     : Array[0..7] Of RGBTripleArrayPtr;
     PixelTG     : Array[0..7] Of RGBTripleArrayPtr;
     PixelTB     : Array[0..7] Of RGBTripleArrayPtr;
     OBMP        : TBitMap;
     WMTemplateR : TBitMap;
     WMTemplateG : TBitMap;
     WMTemplateB : TBitMap;
     WBMPR       : TBitMap;
     WBMPG       : TBitMap;
     WBMPB       : TBitMap;
     BitMap      : TBitMap;
     KeyBitPos   : Byte;
     KeyBitR     : Byte;
     KeyBitG     : Byte;
     KeyBitB     : Byte;
     ShiftCount  : Byte;
     KeyR        : Array[0..15] Of Byte;
     KeyG        : Array[0..15] Of Byte;
     KeyB        : Array[0..15] Of Byte;
     Across      : Integer;
     Down        : Integer;
     AcCount     : Integer;
     DnCount     : Integer;
Begin

 Result := Nil;

 KeyStrRC:=HashWaterMarkKey(KeyStrRC);
 For Z:= 0 to 15 do KeyR[Z]:=Ord(KeyStrRC[Z+1]);

 KeyStrGC:=HashWaterMarkKey(KeyStrGC);
 For Z:= 0 to 15 do KeyG[Z]:=Ord(KeyStrGC[Z+1]);

 KeyStrBC:=HashWaterMarkKey(KeyStrBC);
 For Z:= 0 to 15 do KeyB[Z]:=Ord(KeyStrBC[Z+1]);


 OBMP             := TBitMap.Create;
 OBMP.PixelFormat := pf24Bit;
 OBMP.Width       := OriginalImage.Width  + (OriginalImage.Width  mod 16);
 OBMP.Height      := OriginalImage.Height + (OriginalImage.Height mod 8);
 OBMP.Canvas.Brush.Color:=clBlack;
 OBMP.Canvas.FillRect(Rect(0,0,OBMP.Width,OBMP.Height));
 OBMP.Canvas.Draw(0,0,OriginalImage);

 CreateWaterMarkTemplate(RCWM,WMTemplateR);
 WBMPR             := TBitMap.Create;
 WBMPR.PixelFormat := pf24Bit;
 WBMPR.Width       := OriginalImage.Width  + (OriginalImage.Width  mod 16);
 WBMPR.Height      := OriginalImage.Height + (OriginalImage.Height mod 8);
 WBMPR.Canvas.Brush.Color:=clBlack;
 WBMPR.Canvas.FillRect(Rect(0,0,WBMPR.Width,WBMPR.Height));
 WBMPR.Canvas.Draw(0,0,WMTemplateR);
 WMTemplateR.Destroy;


 CreateWaterMarkTemplate(GCWM,WMTemplateG);
 WBMPG             := TBitMap.Create;
 WBMPG.PixelFormat := pf24Bit;
 WBMPG.Width       := OriginalImage.Width  + (OriginalImage.Width  mod 16);
 WBMPG.Height      := OriginalImage.Height + (OriginalImage.Height mod 8);
 WBMPG.Canvas.Brush.Color:=clBlack;
 WBMPG.Canvas.FillRect(Rect(0,0,WBMPG.Width,WBMPG.Height));
 WBMPG.Canvas.Draw(0,0,WMTemplateG);
 WMTemplateG.Destroy;


 CreateWaterMarkTemplate(BCWM,WMTemplateB);
 WBMPB             := TBitMap.Create;
 WBMPB.PixelFormat := pf24Bit;
 WBMPB.Width       := OriginalImage.Width  + (OriginalImage.Width  mod 16);
 WBMPB.Height      := OriginalImage.Height + (OriginalImage.Height mod 8);
 WBMPB.Canvas.Brush.Color:=clBlack;
 WBMPB.Canvas.FillRect(Rect(0,0,WBMPB.Width,WBMPB.Height));
 WBMPB.Canvas.Draw(0,0,WMTemplateB);
 WMTemplateB.Destroy;


 Across:= OBMP.Width  Div 16;
 Down  := OBMP.Height Div  8;

 For DnCount := 0 to Down-1 do
  Begin

   For z:= 0 to 7 Do
    Begin
     PixelI[z] := OBMP.ScanLine[ (DnCount Shl 3)+z];
     PixelTR[z]:= WBMPR.ScanLine[(DnCount Shl 3)+z];
     PixelTG[z]:= WBMPG.ScanLine[(DnCount Shl 3)+z];
     PixelTB[z]:= WBMPB.ScanLine[(DnCount Shl 3)+z];
    End;
     For y:= 0 to 7 do
      Begin
       For x := 0 to 15 do
        Begin


         {
         KeyBitPos  :=(y Shl 4)+x;
         KeyBitR:=((KeyR[(KeyBitPos Shr 3)] Shr (7-(KeyBitPos And 7))) And $01);
         KeyBitG:=((KeyG[(KeyBitPos Shr 3)] Shr (7-(KeyBitPos And 7))) And $01);
         KeyBitB:=((KeyB[(KeyBitPos Shr 3)] Shr (7-(KeyBitPos And 7))) And $01);
         }

         {
            possible speed up of bit extraction
         }

         KeyBitPos  :=(y Shl 4)+x;
         ShiftCount :=(7-(KeyBitPos And 7));
         KeyBitPos  :=KeyBitPos Shr 3;
         KeyBitR    :=((KeyR[KeyBitPos] Shr ShiftCount) And $01);
         KeyBitG    :=((KeyG[KeyBitPos] Shr ShiftCount) And $01);
         KeyBitB    :=((KeyB[KeyBitPos] Shr ShiftCount) And $01);

          For AcCount:= 0 To Across-1 Do
           Begin

            PixelI[y][x+(AcCount Shl 4)].rgbtRed:= (PixelI[y][x+(AcCount Shl 4)].rgbtRed  And $FE) Or
                                                  ((PixelTR[y][x+(AcCount Shl 4)].rgbtRed Xor KeyBitR) And $01);

            PixelI[y][x+(AcCount Shl 4)].rgbtGreen:= (PixelI[y][x+(AcCount Shl 4)].rgbtGreen  And $FE) Or
                                                    ((PixelTG[y][x+(AcCount Shl 4)].rgbtGreen Xor KeyBitG) And $01);

            PixelI[y][x+(AcCount Shl 4)].rgbtBlue:= (PixelI[y][x+(AcCount Shl 4)].rgbtBlue And $FE) Or
                                                   ((PixelTB[y][x+(AcCount Shl 4)].rgbtBlue Xor KeyBitB) And $01);

           End;

        End;

    End;

  End;

 BitMap             := TBitMap.Create;
 BitMap.PixelFormat := pf24Bit;
 BitMap.Width       := OriginalImage.Width;
 BitMap.Height      := OriginalImage.Height;
 BitMap.Canvas.Draw(0,0,OBMP);
 BitMap.Width       := OriginalImage.Width;
 BitMap.Height      := OriginalImage.Height;
 Result := BitMap;

End;


Function WaterMarkImage.ApplySimpleMD5RedChannel:TBitMap;
Begin
 Result := Nil;
End;


Function WaterMarkImage.ApplySimpleMD5GreenChannel:TBitMap;
Begin
 Result := Nil;
End;


Function WaterMarkImage.ApplySimpleMD5BlueChannel:TBitMap;
Begin
 Result := Nil;
End;


Function WaterMarkImage.ApplySimpleMD5AllChannel:TBitMap;
Begin
 Result := Nil;
End;


Function WaterMarkImage.PaddWaterMarkKey(Key:String):String;
 Var I        : Integer;
     Rounds   : Integer;
     SimpleKey: String;
 Begin
  SimpleKey:='';
  Rounds   :=0;
  For I:= 1 to Length(Key) Do
   Begin
    Rounds:=Rounds+Ord(Key[I]);
   End;
  {Rounds := Rounds Mod Length(Key);}
  For I:= 1 To Rounds Do
   Begin
    SimpleKey := SimpleKey+Key;
   End;
  Result:=SimpleKey;
 End;


Function WaterMarkImage.HashWaterMarkKey(Key:String):String;
Begin
 Result:= MD5Signature(MD5String({PaddWaterMarkKey(Key)}Key));
End;


Procedure WaterMarkImage.TurnOffBit(Bit:Byte; Var BitMap:TBitMap);
Var x,y   : Integer;
    Pixel : RGBTripleArrayPtr;
    RMask : Byte;
    GMask : Byte;
    BMask : Byte;
Begin
 RMask := ALLBITSON;
 GMask := ALLBITSON;
 BMask := ALLBITSON;
  {
 For x:=0 to 7 do

 For y := 0 to Pred(Bitmap.Height) Do
  Begin
   Pixel := Bitmap.ScanLine[y];
   For x := 0 To Pred(Bitmap.Width) Do
    With Pixel[x] Do
     Begin
      rgbtRed   := rgbtRed   And RMask;
      rgbtGreen := rgbtGreen And GMask;
      rgbtBlue  := rgbtBlue  And BMask;
     End
  End;
  }

End;



Begin

End.
